/*
** ssqueue.cpp --- implementation of speech queue.
**
** Copyright (C) 2000, 2001 Bilingual Emacspeak Project <inoue@argv.org>
**
** Last modified: $Date: 2004/05/07 18:03:00 $ by $Author: inoue $
** Keywords: Emacs, Emacspeak, speech, Linux
**
** This file is part of BEP (Bilingual Emacspeak Project) 
** <http://www.argv.org/bep/>
** This file is originally written by Gary Bishop, and modified by BEP
** to make a bilingual, Japanese and English, DECTalk-like speech server 
** running under Windows.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <cstdio>
#include "ssdefs.h"
#include "queue.h"

template <class T>
queue_node<T>::queue_node()
{
  content = NULL;
  next = NULL;
}

template <class T>
queue<T>::queue()
{
  head = NULL;
  tail = NULL;
}

template <class T>
queue<T>::~queue()
{
  clear();
}

template <class T>
queue_node<T> *
queue<T>::push(T *cnt)
{
  queue_node<T> *q = new queue_node<T>();
  q->content = cnt;
  q->next = NULL;
  if (tail != NULL) {
    tail->next = q;
  }
  tail = q;
  if (head == NULL) {
    head = q;
  }
  return q;
}

template <class T>
T *
queue<T>::pop()
{
  T *p;
  queue_node<T> *q;
  q = head;
  if (q != NULL) {
    head = q->next;
    if (head == NULL) tail = NULL;
    p = q->content;
    delete q;
    return p;
  }
  return NULL;
}

template <class T>
int
queue<T>::empty()
{
  return (head == NULL);
}

template <class T>
void
queue<T>::clear()
{
  queue_node<T> *q, *q0;
  q = head;
  while(q != NULL) {
    q0 = q;
    q = q->next;
    delete q0->content;
    delete q0;
  }
  head = tail = NULL;
  return;
}

/* the same as clear() except that this function doesn't
   destroy content of each node. */
template <class T>
void
queue<T>::clear_without_content()
{
  queue_node<T> *q, *q0;
  q = head;
  while(q != NULL) {
    q0 = q;
    q = q->next;
    delete q0;
  }
  head = tail = NULL;
  return;
}

template <class T>
T *
queue<T>::gethead()
{
  if (head == NULL) {
    return NULL;
  }
  return head->content;
}

template <class T>
T *
queue<T>::gettail()
{
  if (tail == NULL) {
    return NULL;
  }
  return tail->content;
}
